; ---------------------------------------------------------------------------
;
; DirectQB BLENDING ROUTINE module
;
; Part of the DirectQB Library version 1.51
; by Angelo Mottola, Enhanced Creations 1998-99
;
; ---------------------------------------------------------------------------

.MODEL medium,basic

.386

.STACK 200h
EXTRN LastError:BYTE
EXTRN PutMode:WORD
EXTRN ClipX1:WORD
EXTRN ClipX2:WORD
EXTRN ClipY1:WORD
EXTRN ClipY2:WORD

EXTRN BMapActive:BYTE
EXTRN BMapSeg:WORD

EXTRN GetLayerSeg:FAR
EXTRN HookInt24:FAR
EXTRN UnhookInt24:FAR


.CODE

; ---------------------------------------------------------------------------
; DQBfilterBox SUB
; purpose:
;   Draws a translucent full box with x1,y1 and x2,y2 as the opposite corners
;   with a given color. Please note that must be x1<x2 and y1<y2; no checks
;   are done and clipping is not supported to speed things up.
; declaration:
;   DECLARE SUB DQBfilterBox(BYVAL Layer,BYVAL x1,BYVAL y1,BYVAL x2,BYVAL y2,
;                       BYVAL Col)
; ---------------------------------------------------------------------------
PUBLIC DQBfilterBox
DQBfilterBox PROC
  ; Stack layout:
  ;
  ; 18  Layer
  ; 16  x1
  ; 14  y1
  ; 12  x2
  ; 10  y2
  ; 08  Col
  ; 06  Basic return segment
  ; 04  Basic return offset
  ; 02  DS
  ; 00  BP
  PUSH DS
  PUSH BP
  MOV BP,SP
  CMP BMapActive,0
  JE EndFilterBox
  MOV BX,[BP+18]
  CALL GetLayerSeg
  MOV ES,CX
  MOV DI,[BP+14]
  MOV BX,DI
  SHL DI,8
  SHL BX,6
  ADD DI,BX
  ADD DI,[BP+16]
  MOV DX,[BP+10]
  SUB DX,[BP+14]
  INC DX
  MOV AX,BMapSeg
  MOV DS,AX
fBoxYloop:
  MOV CX,[BP+12]
  SUB CX,[BP+16]
  INC CX
  PUSH DX
fBoxXloop:
  MOV BL,ES:[DI]
  MOV AX,[BP+8]
  MOV BH,AL
  MOV AL,DS:[BX]
  STOSB
  DEC CX
  JNZ fBoxXloop
  POP DX
  ADD DI,320
  SUB DI,[BP+12]
  ADD DI,[BP+16]
  DEC DI
  DEC DX
  JNZ fBoxYloop
EndFilterBox:
  POP BP
  POP DS
  RET 12
DQBfilterBox ENDP

; ---------------------------------------------------------------------------
; DQBbPut SUB
; purpose:
;   Draws a sprite onto given layer, blending colors using the specified
;   blender map previously built by calling the DQBsetBlenderMap function.
;   This function supports clipping and transparency.
; declaration:
;   DECLARE SUB DQBbPut(BYVAL Layer,BYVAL x,BYVAL y,BYVAL BufferSeg,
;                       BYVAL BufferOff)
; ---------------------------------------------------------------------------
PUBLIC DQBbPut
DQBbPut PROC
  ; Stack layout
  ;
  ; 16  Layer
  ; 14  x
  ; 12  y
  ; 10  Buffer segment
  ; 08  Buffer offset
  ; 06  Basic return segment
  ; 04  Basic return offset
  ; 02  DS
  ; 00  BP
  PUSH DS
  PUSH BP
  MOV BP,SP
  CMP BMapActive,0
  JE EndBPut1
  MOV SI,[BP+08]    ; SI points to the buffer offset
  MOV BX,[BP+16]    ; Gets the layer number...
  CALL GetLayerSeg
  MOV ES,CX         ; ...and sets ES to its segment
  MOV DI,[BP+12]    ; DI contains the y value
  DEC DI
  MOV AX,PutMode    ; Puts the put mode and the clipping box coordinates
  PUSH AX           ; on the stack for later use
  MOV AX,ClipX1
  PUSH AX
  MOV AX,ClipX2
  PUSH AX
  MOV AX,ClipY1
  PUSH AX
  MOV AX,ClipY2
  PUSH AX
  MOV AX,BMapSeg
  MOV GS,AX
  MOV BX,[BP+10]
  MOV DS,BX         ; DS points to the buffer segment
  MOV BX,DI
  SHL DI,8
  SHL BX,6
  ADD DI,BX
  ADD DI,[BP+14]    ; DI points to (x,y-1) on specified layer
  LODSW
  SHR AX,3          ; Divide width by 8 for GET/PUT compatibility
  PUSH AX           ; Gets the sprite width and puts it onto the stack
  LODSW
  MOV AH,AL         ; Sets AH to the height of the sprite
  INC AH
  XOR AL,AL
  MOV BP,SP
  MOV BX,[BP+24]    ; BX holds the value y-1
  DEC BX
  ADD DI,[BP]
  CMP [BP+10],AL    ; Are we in solid put mode?
  JNE bSolidYloop   ; Yes: jump to the solid put loop
bYloop:
  ADD SI,[BP]
  ADD DI,320        ; Adds 320 to DI
  INC BX            ; Next line
  DEC AH
  JZ EndBPut         ; Last sprite line reached
  CMP BX,[BP+4]     ; Are we out of the upper clipping border?
  JL bYloop         ; Yes: increase the line counter
  CMP BX,[BP+2]     ; Are we out of the lower clipping border?
  JG EndBPut        ; Yes: exit DQBput
  SUB DI,[BP]       ; Subtracts the sprite width to DI and SI
  SUB SI,[BP]
  MOV DX,[BP+26]    ; DX holds the current x value
  MOV CX,[BP]       ; Sets CX to the sprite width
bXloop:
  LODSB             ; Gets a pixel from the buffer
  CMP AL,0
  JE bSkipPixel     ; Transparent pixel: skip it!
  CMP DX,[BP+8]
  JL bSkipPixel     ; Out of left layer border: skip it!
  CMP DX,[BP+6]
  JG bSkipPixel     ; Out of right layer border: skip it!
  PUSH BX
  MOV BL,ES:[DI]
  MOV BH,AL
  MOV AL,GS:[BX]
  POP BX
  STOSB             ; All checks done: writes pixel on the layer
  INC DX            ; Next pixel
  DEC CX
  JNZ bXloop       ; Repeats sprite width times
  JMP bYloop        ; Next line
bSkipPixel:
  INC DI            ; Increase DI without writing pixel
  INC DX            ; Next pixel
  DEC CX
  JNZ bXloop       ; Repeats sprite width times
  JMP bYloop        ; Next line
bSolidYloop:
  ADD SI,[BP]
  ADD DI,320        ; Adds 320 to DI
  INC BX            ; Next line
  DEC AH
  JZ EndBPut        ; Last sprite line reached?
  CMP BX,[BP+4]     ; Are we out of the upper clipping border?
  JL bSolidYloop    ; Yes: increase the line counter
  CMP BX,[BP+2]     ; Are we out of the lower clipping border?
  JG EndBPut        ; Yes: exit DQBput
  SUB DI,[BP]       ; Subtracts the sprite width to DI and SI
  SUB SI,[BP]
  MOV DX,[BP+26]    ; DX holds the current x value
  MOV CX,[BP]       ; Sets CX to the sprite width
bSolidXloop:
  LODSB             ; Gets a pixel from the buffer
  CMP DX,[BP+8]
  JL bSkipSolidPixel    ; Out of left layer border: skip it!
  CMP DX,[BP+6]
  JG bSkipSolidPixel    ; Out of right layer border: skip it!
  PUSH BX
  MOV BL,ES:[DI]
  MOV BH,AL
  MOV AL,GS:[BX]
  POP BX
  STOSB             ; All checks done: writes pixel on the layer
  INC DX            ; Next pixel
  DEC CX
  JNZ bSolidXloop  ; Repeats sprite width times
  JMP bSolidYloop   ; Next line
bSkipSolidPixel:
  INC DI            ; Increase DI without writing pixel
  INC DX            ; Next pixel
  DEC CX
  JNZ bSolidXloop  ; Repeats sprite width times
  JMP bSolidYloop   ; Next line
EndBPut:
  ADD SP,12         ; Release stack memory
EndBPut1:
  POP BP
  POP DS
  RET 10
DQBbPut ENDP

; ---------------------------------------------------------------------------
; DQBcreateBMap FUNCTION
; purpose:
;   Allocates memory for the blender map
; declaration:
;   DECLARE FUNCTION DQBcreateBMap()
; ---------------------------------------------------------------------------
PUBLIC DQBcreateBMap
DQBcreateBMap PROC
  CMP BMapActive,1
  JE Error_AlreadyUp
  MOV AH,48h
  MOV BX,4096
  INT 21h
  JC Error_AllocBMap
  MOV BMapSeg,AX
  MOV BMapActive,1
  XOR ECX,ECX
  XOR DI,DI
  MOV ES,AX
CreateBMap:
  MOV ES:[DI],CH
  INC ECX
  INC DI
  CMP ECX,65536
  JL CreateBMap
  XOR AX,AX
  RET
Error_AlreadyUp:
  MOV AX,2          ; Error 2: Blender map already created
  MOV LastError,6
  RET
Error_AllocBMap:
  MOV AX,1          ; Error 1: Not enough free conventional memory
  MOV LastError,7
  RET
DQBcreateBMap ENDP

; ---------------------------------------------------------------------------
; DQBsetBMap SUB
; purpose:
;   Sets a color entry on the blender map (if active)
; declaration:
;   DECLARE SUB DQBsetBMap(BYVAL ForeCol,BYVAL BackCol,BYVAL NewCol)
; ---------------------------------------------------------------------------
PUBLIC DQBsetBMap
DQBsetBMap PROC
  ; Stack layout:
  ;
  ; 10  ForeCol
  ; 08  BackCol
  ; 06  NewCol
  ; 04  Basic return segment
  ; 02  Basic return offset
  ; 00  BP
  PUSH BP
  MOV BP,SP
  CMP BMapActive,0
  JE EndSetBMap
  MOV AX,BMapSeg
  MOV ES,AX
  MOV AX,[BP+10]
  MOV BX,[BP+8]
  MOV BH,AL
  MOV AL,[BP+6]
  MOV ES:[BX],AL
EndSetBMap:
  POP BP
  RET 6
DQBsetBMap ENDP

; ---------------------------------------------------------------------------
; DQBgetBMap FUNCTION
; purpose:
;   Returns a color entry on the blender map (if active)
; declaration:
;   DECLARE FUNCTION DQBgetBMap(BYVAL ForeCol,BYVAL BackCol)
; ---------------------------------------------------------------------------
PUBLIC DQBgetBMap
DQBgetBMap PROC
  ; Stack layout:
  ;
  ; 08  ForeCol
  ; 06  BackCol
  ; 04  Basic return segment
  ; 02  Basic return offset
  ; 00  BP
  PUSH BP
  MOV BP,SP
  MOV AX,-1
  CMP BMapActive,0
  JE EndGetBMap
  MOV AX,BMapSeg
  MOV ES,AX
  MOV AX,[BP+8]
  MOV BX,[BP+6]
  MOV BH,AL
  XOR AH,AH
  MOV AL,ES:[BX]
EndGetBMap:
  POP BP
  RET 4
DQBgetBMap ENDP

; ---------------------------------------------------------------------------
; DQBloadBMap FUNCTION
; purpose:
;   Loads the blender map from a specified file
; declaration:
;   DECLARE FUNCTION xDQBloadBMap(BYVAL FileSeg,BYVAL FileOff)
;   DECLARE FUNCTION DQBloadBMap(FileName AS STRING)
; ---------------------------------------------------------------------------
PUBLIC xDQBloadBMap
xDQBloadBMap PROC
  ; Stack layout:
  ;
  ; 10  FileSeg
  ; 08  FileOff
  ; 06  Basic return segment
  ; 04  Basic return offset
  ; 02  DS
  ; 00  BP
  PUSH DS
  PUSH BP
  MOV BP,SP
  CALL HookInt24
  CMP BMapActive,0
  JE Error_NoBMapLoad
  MOV AX,BMapSeg
  MOV ES,AX
  MOV AX,03D00h
  MOV BX,[BP+10]
  MOV DS,BX
  MOV DX,[BP+8]
  INT 21h
  JC Error_OpenBMapLoad
  MOV BX,AX
  MOV AX,ES
  MOV DS,AX
  MOV AH,03Fh
  XOR DX,DX
  MOV CX,08000h
  INT 21h
  JC Error_ReadBMap
  MOV AH,03Fh
  MOV DX,08000h
  MOV CX,08000h
  INT 21h
  JC Error_ReadBMap
  MOV AH,03Eh
  INT 21h
  XOR AX,AX
  JMP EndLoadBMap
Error_NoBMapLoad:
  MOV AH,03Eh
  INT 21h
  MOV AX,1          ; Error 1: Blender map not yet created
  MOV LastError,8
  JMP EndLoadBMap
Error_OpenBMapLoad:
  MOV AH,03Eh
  INT 21h
  MOV AX,2          ; Error 2: Unable to open file or file does not exist
  MOV LastError,4
  JMP EndLoadBMap
Error_ReadBMap:
  MOV AH,03Eh
  INT 21h
  MOV AX,3          ; Error 3: General file read error
  MOV LastError,5
EndLoadBMap:
  CALL UnhookInt24
  POP BP
  POP DS
  RET 4
xDQBloadBMap ENDP

; ---------------------------------------------------------------------------
; DQBsaveBMap FUNCTION
; purpose:
;   Saves the blender map into a specified file
; declaration:
;   DECLARE FUNCTION xDQBsaveBMap(BYVAL FileSeg,BYVAL FileOff)
;   DECLARE FUNCTION DQBsaveBMap(FileName AS STRING)
; ---------------------------------------------------------------------------
PUBLIC xDQBsaveBMap
xDQBsaveBMap PROC
  ; Stack layout:
  ;
  ; 10  FileSeg
  ; 08  FileOff
  ; 06  Basic return segment
  ; 04  Basic return offset
  ; 02  DS
  ; 00  BP
  PUSH DS
  PUSH BP
  MOV BP,SP
  CALL HookInt24
  CMP BMapActive,0
  JE Error_NoBMapSave
  MOV AX,BMapSeg
  MOV ES,AX
  MOV AH,03Ch
  MOV CX,020h
  MOV BX,[BP+10]
  MOV DS,BX
  MOV DX,[BP+8]
  INT 21h
  JC Error_OpenBMapSave
  MOV BX,AX
  MOV AX,ES
  MOV DS,AX
  MOV AH,040h
  XOR DX,DX
  MOV CX,08000h
  INT 21h
  JC Error_WriteBMap
  MOV AH,040h
  MOV DX,08000h
  MOV CX,08000h
  INT 21h
  JC Error_WriteBMap
  MOV AH,03Eh
  INT 21h
  XOR AX,AX
  JMP EndSaveBMap
Error_NoBMapSave:
  MOV AH,03Eh
  INT 21h
  MOV AX,1          ; Error 1: Blender map not yet created
  MOV LastError,8
  JMP EndSaveBMap
Error_OpenBMapSave:
  MOV AH,03Eh
  INT 21h
  MOV AX,2          ; Error 2: Unable to create file
  MOV LastError,9
  JMP EndSaveBMap
Error_WriteBMap:
  MOV AH,03Eh
  INT 21h
  MOV AX,3          ; Error 3: General file write error
  MOV LastError,10
EndSaveBMap:
  CALL UnhookInt24
  POP BP
  POP DS
  RET 4
xDQBsaveBMap ENDP

; ---------------------------------------------------------------------------
; DQBremoveBMap SUB
; purpose:
;   Deallocates memory used by the blender map
; declaration:
;   DECLARE FUNCTION DQBremoveBMap()
; ---------------------------------------------------------------------------
PUBLIC DQBremoveBMap
DQBremoveBMap PROC
  CMP BMapActive,0
  JE EndRemoveBMap
  MOV AH,49h
  MOV BX,BMapSeg
  MOV ES,BX
  INT 21h
  MOV BMapActive,0
EndRemoveBMap:
  RET
DQBremoveBMap ENDP


END
